/*
 * Copyright (c) 2013, 2014, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.oracle.truffle.r.nodes.function;

import com.oracle.truffle.r.nodes.*;
import com.oracle.truffle.r.runtime.*;
import com.oracle.truffle.r.runtime.data.*;

import com.oracle.truffle.api.*;
import com.oracle.truffle.api.frame.*;
import com.oracle.truffle.api.nodes.*;

public final class InlinedFunctionNode extends CallNode implements InlinedCallSite {

    @Child private CallArgumentsNode arguments;
    @Child private RNode body;

    private final FrameFactory factory;
    private final REnvironment descriptor;
    private final String description;

    public InlinedFunctionNode(FrameFactory factory, REnvironment descriptor, RNode body, String description, CallArgumentsNode arguments) {
        this.factory = factory;
        this.descriptor = descriptor;
        this.description = description;
        this.arguments = adoptChild(arguments);
        this.body = adoptChild(body);
    }

    public String getDescription() {
        return description;
    }

    @Override
    public Object execute(VirtualFrame frame, RFunction function) {
        VirtualFrame childFrame = factory.create(descriptor.getDescriptor(), frame.getCaller(), createArguments(frame, function));
        return body.execute(childFrame);
    }

    private RArguments createArguments(VirtualFrame frame, RFunction function) {
        return RArguments.create(function, arguments.executeArray(frame));
    }

    @Override
    public CallTarget getCallTarget() {
        return ((CachedCallNode) getParent()).getCallTarget();
    }
}
